import static org.junit.Assert.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.*;

import model.HttpClient;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.Mockito;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;

/*
  Zgodnie z wymogami projektu - trzy testy mockujace przygotowane dla klasy HttpClient.
*/

public class HttpClientTest {


    @Test
    void testExecuteGetRequest() throws Exception {

        // Dowolny adres URL
        String urlString = "www.example.com";

        // Przykladowy response
        String expectedResponse = "{\"coord\":{\"lon\":44.34,\"lat\":10.99},\"weather\":" +
                "[{\"id\":804,\"main\":\"Clouds\",\"description\":\"overcast clouds\"" +
                ",\"icon\":\"04n\"}],\"base\":\"stations\",\"main\":{\"temp\":299.46,\"" +
                "feels_like\":299.46,\"temp_min\":299.46,\"temp_max\":299.46,\"pressure\"" +
                ":1014,\"humidity\":66,\"sea_level\":1014,\"grnd_level\":1014},\"" +
                "visibility\":10000,\"wind\":{\"speed\":4.89,\"deg\":78,\"gust\":5.26}," +
                "\"clouds\":{\"all\":88},\"dt\":1701977972,\"sys\":{\"country\":\"SO\"," +
                "\"sunrise\":1701918532,\"sunset\":1701959952},\"timezone\":10800,\"id\"" +
                ":54746,\"name\":\"Lughaye\",\"cod\":200}\n";

        // Przygotowanie mockow
        CloseableHttpClient mockHttpClient = Mockito.mock(CloseableHttpClient.class);
        org.apache.http.HttpEntity mockEntity = Mockito.mock(org.apache.http.HttpEntity.class);
        CloseableHttpResponse mockResponse = Mockito.mock(CloseableHttpResponse.class);
        when(mockResponse.getEntity()).thenReturn(mockEntity);
        when(mockEntity.getContent()).thenReturn(new ByteArrayInputStream(expectedResponse.getBytes(StandardCharsets.UTF_8)));
        when(mockEntity.getContentLength()).thenReturn((long) expectedResponse.length());

        // Przygotowujemy odpowiedź do zwrócenia przez mock
        when(mockHttpClient.execute(any(HttpGet.class))).thenReturn(mockResponse);

        // Instancja testowanej klasy z przekazaniem zmockowanego klienta
        HttpClient httpClient = new HttpClient(mockHttpClient);

        // Act & Assert
        String result = httpClient.executeGetRequest(urlString);
        assertEquals(expectedResponse, result);
    }

    @Test
    void testExecuteGetRequest_EmptyResponse() throws IOException {

        // Dowolny adres url
        String urlString = "www.example.com";

        // Przygotowanie mockow
        CloseableHttpClient mockHttpClient = mock(CloseableHttpClient.class);
        CloseableHttpResponse mockHttpResponse = mock(CloseableHttpResponse.class);
        when(mockHttpClient.execute(ArgumentMatchers.any(HttpGet.class))).thenReturn(mockHttpResponse);
        when(mockHttpResponse.getEntity()).thenReturn(null);

        // Instancja testowanej klasy z przekazaniem zmockowanego klienta
        HttpClient httpClient = new HttpClient(mockHttpClient);

        // Act & Assert
        String result = httpClient.executeGetRequest(urlString);

        assertEquals("", result);
    }
    @Test
    void testExecuteGetRequest_ClientProtocolException() throws IOException {

        // Przygotowanie mockow
        CloseableHttpClient mockHttpClient = mock(CloseableHttpClient.class);
        when(mockHttpClient.execute(ArgumentMatchers.any(HttpGet.class))).thenThrow(new ClientProtocolException("Mocked ClientProtocolException"));

        // Instancja testowanej klasy z przekazaniem zmockowanego klienta
        HttpClient httpClient = new HttpClient(mockHttpClient);

        // Act & Assert
        RuntimeException runtimeException = assertThrows(RuntimeException.class, () ->
                httpClient.executeGetRequest("http://example.com")
        );
        assertEquals("org.apache.http.client.ClientProtocolException: Mocked ClientProtocolException", runtimeException.getMessage());
    }
}
